@@ -14,8 +14,6 @@
 #endif
 #ifdef WIN32
 #include <winsock.h>
-#elif defined(__SYMBIAN32__)
-#define bzero(b,len) memset(b,0,len)
 #endif
 #ifdef HAVE_SIGNAL
 #include <signal.h>
@@ -157,11 +155,7 @@ static struct re global_re = {
 
 static void poll_close(struct re *re);
 
-#ifdef SOLARIS
-static pthread_once_t pt_once = {PTHREAD_ONCE_INIT};
-#else
 static pthread_once_t pt_once = PTHREAD_ONCE_INIT;
-#endif
 static pthread_key_t  pt_key;
 
 
@@ -790,7 +784,7 @@ static int fd_poll(struct re *re)
 				flags |= FD_READ;
 			if (re->events[i].events & EPOLLOUT)
 				flags |= FD_WRITE;
-			if (re->events[i].events & EPOLLERR)
+			if (re->events[i].events & (EPOLLERR|EPOLLHUP))
 				flags |= FD_EXCEPT;
 
 			if (!flags) {
@@ -967,13 +961,6 @@ int re_main(re_signal_h *signalh)
 
 	re->polling = true;
 
-#ifdef HAVE_ACTSCHED
-	if (METHOD_ACTSCHED == re->method) {
-		err = actsched_start();
-		goto out;
-	}
-#endif
-
 	re_lock(re);
 	for (;;) {
 
@@ -1022,12 +1009,6 @@ void re_cancel(void)
 	struct re *re = re_get();
 
 	re->polling = false;
-
-#ifdef HAVE_ACTSCHED
-	if (METHOD_ACTSCHED == re->method) {
-		actsched_stop();
-	}
-#endif
 }
 
 
@@ -1093,10 +1074,6 @@ int poll_method_set(enum poll_method method)
 			return EINVAL;
 		break;
 #endif
-#ifdef HAVE_ACTSCHED
-	case METHOD_ACTSCHED:
-		break;
-#endif
 #ifdef HAVE_KQUEUE
 	case METHOD_KQUEUE:
 		break;
