@@ -4,11 +4,13 @@
  * Copyright (C) 2010 Creytiv.com
  */
 #include <string.h>
-#define OPENSSL_NO_KRB5 1
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/rsa.h>
 #include <openssl/bn.h>
+#include <openssl/evp.h>
+#include <openssl/x509.h>
+#include <openssl/x509v3.h>
 #include <re_types.h>
 #include <re_fmt.h>
 #include <re_mem.h>
@@ -23,6 +25,12 @@
 #include "tls.h"
 
 
+/* also defined by wincrypt.h */
+#ifdef WIN32
+#undef X509_NAME
+#endif
+
+
 #define DEBUG_MODULE "tls"
 #define DEBUG_LEVEL 5
 #include <re_dbg.h>
@@ -66,6 +74,19 @@ static int password_cb(char *buf, int size, int rwflag, void *userdata)
 }
 
 
+static int keytype2int(enum tls_keytype type)
+{
+	switch (type) {
+	case TLS_KEYTYPE_EC:
+		return EVP_PKEY_EC;
+	case TLS_KEYTYPE_RSA:
+		return EVP_PKEY_RSA;
+	default:
+		return EVP_PKEY_NONE;
+	}
+}
+
+
 /**
  * Allocate a new TLS context
  *
@@ -97,7 +118,13 @@ int tls_alloc(struct tls **tlsp, enum tls_method method, const char *keyfile,
 
 #ifdef USE_OPENSSL_DTLS
 	case TLS_METHOD_DTLSV1:
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && \
+	!defined(LIBRESSL_VERSION_NUMBER)
+
+		tls->ctx = SSL_CTX_new(DTLS_method());
+#else
 		tls->ctx = SSL_CTX_new(DTLSv1_method());
+#endif
 		break;
 
 #ifdef SSL_OP_NO_DTLSv1_2
@@ -108,7 +135,13 @@ int tls_alloc(struct tls **tlsp, enum tls_method method, const char *keyfile,
 		break;
 
 	case TLS_METHOD_DTLSV1_2:
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && \
+	!defined(LIBRESSL_VERSION_NUMBER)
+
+		tls->ctx = SSL_CTX_new(DTLS_method());
+#else
 		tls->ctx = SSL_CTX_new(DTLSv1_2_method());
+#endif
 		break;
 #endif
 
@@ -176,18 +209,18 @@ int tls_alloc(struct tls **tlsp, enum tls_method method, const char *keyfile,
  * Set default locations for trusted CA certificates
  *
  * @param tls    TLS Context
- * @param capath Path to CA certificates
+ * @param cafile PEM file with CA certificates
  *
  * @return 0 if success, otherwise errorcode
  */
-int tls_add_ca(struct tls *tls, const char *capath)
+int tls_add_ca(struct tls *tls, const char *cafile)
 {
-	if (!tls || !capath)
+	if (!tls || !cafile)
 		return EINVAL;
 
 	/* Load the CAs we trust */
-	if (!(SSL_CTX_load_verify_locations(tls->ctx, capath, 0))) {
-		DEBUG_WARNING("Can't read CA list: %s\n", capath);
+	if (!(SSL_CTX_load_verify_locations(tls->ctx, cafile, NULL))) {
+		DEBUG_WARNING("Can't read CA file: %s\n", cafile);
 		ERR_clear_error();
 		return EINVAL;
 	}
@@ -196,6 +229,90 @@ int tls_add_ca(struct tls *tls, const char *capath)
 }
 
 
+/**
+ * Set SSL verification of the certificate purpose
+ *
+ * @param tls     TLS Context
+ * @param purpose Certificate purpose as string
+ *
+ * @return int    0 if success, errorcode otherwise
+ */
+int tls_set_verify_purpose(struct tls *tls, const char *purpose)
+{
+	int err;
+	int i;
+	X509_PURPOSE *xptmp;
+
+	if (!tls || !purpose)
+		return EINVAL;
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	i = X509_PURPOSE_get_by_sname(purpose);
+#else
+	i = X509_PURPOSE_get_by_sname((char *) purpose);
+#endif
+
+	if (i < 0)
+		return EINVAL;
+
+	/* purpose index -> purpose object */
+	/* purpose object -> purpose value */
+	xptmp = X509_PURPOSE_get0(i);
+	i = X509_PURPOSE_get_id(xptmp);
+	err = SSL_CTX_set_purpose(tls->ctx, i);
+
+	return err == 1 ? 0 : EINVAL;
+}
+
+
+/**
+ * Set SSL verification of hostname
+ *
+ * @param tc       TLS Connection
+ * @param hostname Certificate hostname
+ *
+ * @return int     0 if success, errorcode otherwise
+ */
+int tls_peer_set_verify_host(struct tls_conn *tc, const char *hostname)
+{
+	int err = 0;
+
+	if (!tc)
+		return EINVAL;
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	err = SSL_set1_host(tc->ssl, hostname);
+#else
+	DEBUG_WARNING("verify hostname needs openssl version 1.1.0\n");
+	return ENOSYS;
+#endif
+
+	return err == 1 ? 0 : EINVAL;
+}
+
+
+/**
+ * Convert string hostname to pl hostname
+ *
+ * @param tls_hostname Certificate hostname as string
+ * @param hostname     Certificate hostname as pl
+ *
+ * @return int         0 if success, errorcode otherwise
+ */
+int tls_set_hostname(char *tls_hostname, const struct pl *hostname)
+{
+	if (!tls_hostname || !hostname)
+		return EINVAL;
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+	DEBUG_WARNING("verify hostname needs openssl version 1.1.0\n");
+	return ENOSYS;
+#endif
+
+	return pl_strdup(&tls_hostname, hostname);
+}
+
+
 /**
  * Generate and set selfsigned certificate on TLS context
  *
@@ -205,6 +322,12 @@ int tls_add_ca(struct tls *tls, const char *capath)
  * @return 0 if success, otherwise errorcode
  */
 int tls_set_selfsigned(struct tls *tls, const char *cn)
+{
+	return tls_set_selfsigned_rsa(tls, cn, 1024);
+}
+
+
+int tls_set_selfsigned_rsa(struct tls *tls, const char *cn, size_t bits)
 {
 	X509_NAME *subj = NULL;
 	EVP_PKEY *key = NULL;
@@ -225,7 +348,7 @@ int tls_set_selfsigned(struct tls *tls, const char *cn)
 		goto out;
 
 	BN_set_word(bn, RSA_F4);
-	if (!RSA_generate_key_ex(rsa, 1024, bn, NULL))
+	if (!RSA_generate_key_ex(rsa, (int)bits, bn, NULL))
 		goto out;
 
 	key = EVP_PKEY_new();
@@ -258,9 +381,15 @@ int tls_set_selfsigned(struct tls *tls, const char *cn)
 	    !X509_set_subject_name(cert, subj))
 		goto out;
 
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	if (!X509_gmtime_adj(X509_getm_notBefore(cert), -3600*24*365) ||
+	    !X509_gmtime_adj(X509_getm_notAfter(cert),   3600*24*365*10))
+		goto out;
+#else
 	if (!X509_gmtime_adj(X509_get_notBefore(cert), -3600*24*365) ||
 	    !X509_gmtime_adj(X509_get_notAfter(cert),   3600*24*365*10))
 		goto out;
+#endif
 
 	if (!X509_set_pubkey(cert, key))
 		goto out;
@@ -310,55 +439,63 @@ int tls_set_selfsigned(struct tls *tls, const char *cn)
 /**
  * Set the certificate and private key on a TLS context
  *
- * @param tls TLS Context
- * @param pem Certificate and private key in PEM format
- * @param len Length of PEM string
+ * @param tls      TLS Context
+ * @param cert     Certificate in PEM format
+ * @param len_cert Length of certificate PEM string
+ * @param key      Private key in PEM format, will be read from cert if NULL
+ * @param len_key  Length of private key PEM string
  *
  * @return 0 if success, otherwise errorcode
  */
-int tls_set_certificate(struct tls *tls, const char *pem, size_t len)
+int tls_set_certificate_pem(struct tls *tls, const char *cert, size_t len_cert,
+			    const char *key, size_t len_key)
 {
 	BIO *bio = NULL, *kbio = NULL;
-	X509 *cert = NULL;
-	RSA *rsa = NULL;
+	X509 *x509 = NULL;
+	EVP_PKEY *pkey = NULL;
 	int r, err = ENOMEM;
 
-	if (!tls || !pem || !len)
+	if (!tls || !cert || !len_cert || (key && !len_key))
 		return EINVAL;
 
-	bio  = BIO_new_mem_buf((char *)pem, (int)len);
-	kbio = BIO_new_mem_buf((char *)pem, (int)len);
+	if (!key) {
+		key = cert;
+		len_key = len_cert;
+	}
+
+	bio  = BIO_new_mem_buf((char *)cert, (int)len_cert);
+	kbio = BIO_new_mem_buf((char *)key, (int)len_key);
 	if (!bio || !kbio)
 		goto out;
 
-	cert = PEM_read_bio_X509(bio, NULL, 0, NULL);
-	rsa = PEM_read_bio_RSAPrivateKey(kbio, NULL, 0, NULL);
-	if (!cert || !rsa)
+	x509 = PEM_read_bio_X509(bio, NULL, 0, NULL);
+	pkey = PEM_read_bio_PrivateKey(kbio, NULL, 0, NULL);
+	if (!x509 || !pkey)
 		goto out;
 
-	r = SSL_CTX_use_certificate(tls->ctx, cert);
+	r = SSL_CTX_use_certificate(tls->ctx, x509);
 	if (r != 1)
 		goto out;
 
-	r = SSL_CTX_use_RSAPrivateKey(tls->ctx, rsa);
+	r = SSL_CTX_use_PrivateKey(tls->ctx, pkey);
 	if (r != 1) {
-		DEBUG_WARNING("set_certificate: use_RSAPrivateKey failed\n");
+		DEBUG_WARNING("set_certificate_pem: use_PrivateKey failed\n");
 		goto out;
 	}
 
 	if (tls->cert)
 		X509_free(tls->cert);
 
-	tls->cert = cert;
-	cert = NULL;
+	tls->cert = x509;
+	x509 = NULL;
 
 	err = 0;
 
  out:
-	if (cert)
-		X509_free(cert);
-	if (rsa)
-		RSA_free(rsa);
+	if (x509)
+		X509_free(x509);
+	if (pkey)
+		EVP_PKEY_free(pkey);
 	if (bio)
 		BIO_free(bio);
 	if (kbio)
@@ -370,6 +507,94 @@ int tls_set_certificate(struct tls *tls, const char *pem, size_t len)
 }
 
 
+/**
+ * Set the certificate and private key on a TLS context
+ *
+ * @param tls      TLS Context
+ * @param keytype  Private key type
+ * @param cert     Certificate in DER format
+ * @param len_cert Length of certificate DER bytes
+ * @param key      Private key in DER format, will be read from cert if NULL
+ * @param len_key  Length of private key DER bytes
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int tls_set_certificate_der(struct tls *tls, enum tls_keytype keytype,
+			    const uint8_t *cert, size_t len_cert,
+			    const uint8_t *key, size_t len_key)
+{
+	const uint8_t *buf_cert;
+	X509 *x509 = NULL;
+	EVP_PKEY *pkey = NULL;
+	int r, type, err = ENOMEM;
+
+	if (!tls || !cert || !len_cert || (key && !len_key))
+		return EINVAL;
+
+	type = keytype2int(keytype);
+	if (type == EVP_PKEY_NONE)
+		return EINVAL;
+
+	buf_cert = cert;
+
+	x509 = d2i_X509(NULL, &buf_cert, len_cert);
+	if (!x509)
+		goto out;
+
+	if (!key) {
+		key = buf_cert;
+		len_key = len_cert - (buf_cert - cert);
+	}
+
+	pkey = d2i_PrivateKey(type, NULL, &key, len_key);
+	if (!pkey)
+		goto out;
+
+	r = SSL_CTX_use_certificate(tls->ctx, x509);
+	if (r != 1)
+		goto out;
+
+	r = SSL_CTX_use_PrivateKey(tls->ctx, pkey);
+	if (r != 1) {
+		DEBUG_WARNING("set_certificate_der: use_PrivateKey failed\n");
+		goto out;
+	}
+
+	if (tls->cert)
+		X509_free(tls->cert);
+
+	tls->cert = x509;
+	x509 = NULL;
+
+	err = 0;
+
+ out:
+	if (x509)
+		X509_free(x509);
+	if (pkey)
+		EVP_PKEY_free(pkey);
+	if (err)
+		ERR_clear_error();
+
+	return err;
+}
+
+
+/**
+ * Set the certificate and private key on a TLS context
+ *
+ * @param tls TLS Context
+ * @param pem Certificate and private key in PEM format
+ * @param len Length of PEM string
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int tls_set_certificate(struct tls *tls, const char *pem, size_t len)
+{
+	return tls_set_certificate_pem(tls, pem, len, NULL, 0);
+}
+
+
 static int verify_handler(int ok, X509_STORE_CTX *ctx)
 {
 	(void)ok;
@@ -607,6 +832,22 @@ int tls_srtp_keyinfo(const struct tls_conn *tc, enum srtp_suite *suite,
 		salt_size = 14;
 		break;
 
+#ifdef SRTP_AEAD_AES_128_GCM
+	case SRTP_AEAD_AES_128_GCM:
+		*suite = SRTP_AES_128_GCM;
+		key_size  = 16;
+		salt_size = 12;
+		break;
+#endif
+
+#ifdef SRTP_AEAD_AES_256_GCM
+	case SRTP_AEAD_AES_256_GCM:
+		*suite = SRTP_AES_256_GCM;
+		key_size  = 32;
+		salt_size = 12;
+		break;
+#endif
+
 	default:
 		return ENOSYS;
 	}
@@ -730,3 +971,65 @@ int tls_set_servername(struct tls_conn *tc, const char *servername)
 
 	return 0;
 }
+
+
+/**
+ * Enable verification of server certificate and hostname
+ *
+ * @param tc   TLS Connection
+ * @param host Server hostname
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int tls_set_verify_server(struct tls_conn *tc, const char *host)
+{
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && \
+	!defined(LIBRESSL_VERSION_NUMBER)
+
+	if (!tc || !host)
+		return EINVAL;
+
+	SSL_set_hostflags(tc->ssl, X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS);
+	if (!SSL_set1_host(tc->ssl, host)) {
+		ERR_clear_error();
+		return EPROTO;
+	}
+
+	SSL_set_verify(tc->ssl, SSL_VERIFY_PEER, NULL);
+
+	return 0;
+#else
+	(void)tc;
+	(void)host;
+
+	return ENOSYS;
+#endif
+}
+
+
+static int print_error(const char *str, size_t len, void *unused)
+{
+	(void)unused;
+	DEBUG_WARNING("%b", str, len);
+
+	return 1;
+}
+
+
+void tls_flush_error(void)
+{
+	ERR_print_errors_cb(print_error, NULL);
+}
+
+
+/**
+ * Get the backend-specific (OpenSSL) context
+ *
+ * @param tls  Generic TLS Context
+ *
+ * @return OpenSSL context
+ */
+struct ssl_ctx_st *tls_openssl_context(const struct tls *tls)
+{
+	return tls ? tls->ctx : NULL;
+}
